/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

#include "../std.h"
#include "ImageCache.h"
#include "MD5.h"
#include <mavsprintf.h>
#include <MAUI/Image.h>

ImageCache::ImageCache()
{
  mDownloader = new Downloader();
  mDownloader->addDownloadListener(this);
  mIsBusy = false;
}

ImageCache::~ImageCache()
{
  if(mDownloader->isDownloading())
    mDownloader->cancelDownloading();
  delete mDownloader;
}

void ImageCache::request(ImageCacheRequest* req)
{
  //Check that there is a valid listener
  if(req->getRequester() != NULL)
  {
	//Add the request to the queue
    mRequests.add(req);

    //Process the queue
    process();
  }
}

void ImageCache::process()
{
	//Check to see if the cache can process this request at this time
	if(mIsBusy) return;

	//Check to see if there are any outstanding requests
	if(mRequests.size() == 0)
	return;

	//Set the mIsBusy flag, so we don't try to do too much at once
	mIsBusy = true;

	//Get the next image request from the queue
    mNextRequest = mRequests[0];

    //Clear any existing fingerprint
    mMD5url.clear();

    //If there is a request, get the MD5 fingerprint of the url
    if(mNextRequest != NULL)
    	mMD5url = md5(mNextRequest->getUrl()).c_str();

	//See if the image is already loaded
	HashMap<String, MAHandle>::Iterator local = mHandles.find(mMD5url);
	if(local != mHandles.end())
	{
		//Found in loaded cache
		returnImage((*local).second);
		return;
	}

	mCacheImage = maCreatePlaceholder();

	MAHandle store = maOpenStore(mMD5url.c_str(), 0);
	if(store != STERR_NONEXISTENT)
	{
		//Found in cache
		maReadStore(store, mCacheImage);
		maCloseStore(store, 0);

		mHandles.insert(mMD5url, mCacheImage);

		returnImage(mCacheImage);
		return;
	}
	else
	{
		//Not in memory, and not in storage
		if(mDownloader->isDownloading())
			return;

		mDownloader->beginDownloading(mNextRequest->getUrl().c_str());
	}
}

void ImageCache::finishedDownloading(Downloader* downloader, MAHandle data)
{
	//Save to storage
	MAHandle store = maOpenStore(mMD5url.c_str(), MAS_CREATE_IF_NECESSARY);
	maWriteStore(store, data);
	maCloseStore(store, 0);

	returnImage(data);
}

void ImageCache::notifyProgress(Downloader* downloader, int d, int t)
{
}

void ImageCache::downloadCancelled(Downloader* downloader)
{
}

void ImageCache::error(Downloader* downloader, int errorCode)
{
}

void ImageCache::returnImage(MAHandle i)
{
	//Save to memory
	mHandles.insert(mMD5url, i);

	//Inform listener
	MAHandle imageh = maCreatePlaceholder();
	maCreateImageFromData(imageh, i, 0, maGetDataSize(i));
	mNextRequest->getRequester()->imageReady(imageh);

	delete mNextRequest;
	mRequests.remove(0);

	mIsBusy = false;
	process();
}
